@@ -6,6 +6,7 @@
 #include "qserialport_p.h"
 #include "qserialportinfo_p.h"
 
+#include <QtCore/qdeadlinetimer.h>
 #include <QtCore/qelapsedtimer.h>
 #include <QtCore/qmap.h>
 #include <QtCore/qsocketnotifier.h>
@@ -19,7 +20,7 @@
 #include <sys/time.h>
 #include <unistd.h>
 
-#ifdef Q_OS_OSX
+#ifdef Q_OS_MACOS
 #if defined(MAC_OS_X_VERSION_10_4) && (MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_4)
 #include <IOKit/serial/ioss.h>
 #endif
@@ -614,7 +615,7 @@ bool QSerialPortPrivate::setCustomBaudRate(qint32 baudRate, QSerialPort::Directi
     return setStandardBaudRate(B38400, directions);
 }
 
-#elif defined(Q_OS_OSX)
+#elif defined(Q_OS_MACOS)
 
 bool QSerialPortPrivate::setCustomBaudRate(qint32 baudRate, QSerialPort::Directions directions)
 {
@@ -750,7 +751,7 @@ bool QSerialPortPrivate::readNotification()
 
     buffer.chop(bytesToRead - qMax(readBytes, qint64(0)));
 
-    if (readBytes <= 0) {
+    if (readBytes < 0) {
         QSerialPortErrorInfo error = getSystemError();
         if (error.errorCode != QSerialPort::ResourceError)
             error.errorCode = QSerialPort::ReadError;
@@ -758,6 +759,8 @@ bool QSerialPortPrivate::readNotification()
             setReadNotificationEnabled(false);
         setError(error);
         return false;
+    } else if (readBytes == 0) {
+        return false;
     }
 
     newBytes = buffer.size() - newBytes;
@@ -849,6 +852,9 @@ inline bool QSerialPortPrivate::initialize(QIODevice::OpenMode mode)
     if (mode & QIODevice::ReadOnly)
         setReadNotificationEnabled(true);
 
+    // flush IO buffers
+    clear(QSerialPort::AllDirections);
+
     return true;
 }
 
@@ -911,7 +917,7 @@ QSerialPortErrorInfo QSerialPortPrivate::getSystemError(int systemErrorCode) con
     case EBADF:
         error.errorCode = QSerialPort::ResourceError;
         break;
-#ifdef Q_OS_OSX
+#ifdef Q_OS_MACOS
     case ENXIO:
         error.errorCode = QSerialPort::ResourceError;
         break;
@@ -992,7 +998,7 @@ bool QSerialPortPrivate::waitForReadOrWrite(bool *selectForRead, bool *selectFor
     if (checkWrite)
         pfd.events |= POLLOUT;
 
-    const int ret = qt_poll_msecs(&pfd, 1, msecs);
+    const int ret = qt_safe_poll(&pfd, 1, QDeadlineTimer(msecs));
     if (ret < 0) {
         setError(getSystemError());
         return false;
@@ -1059,7 +1065,11 @@ qint64 QSerialPortPrivate::writePerChar(const char *data, qint64 maxSize)
         par ^= parity == QSerialPort::MarkParity;
         if (par ^ (tio.c_cflag & PARODD)) { // Need switch parity mode?
             tio.c_cflag ^= PARODD;
-            flush(); //force sending already buffered data, because setTermios(&tio); cleares buffers
+            //force sending already buffered data, because setTermios(&tio); cleares buffers
+            if (::tcdrain(descriptor) == -1) {
+                setError(getSystemError());
+                break;
+            }
             //todo: add receiving buffered data!!!
             if (!setTermios(&tio))
                 break;
